package meerkat.pollingstation;

import com.google.common.util.concurrent.FutureCallback;
import meerkat.protobuf.PollingStation;
import meerkat.rest.ProtobufMessageBodyReader;
import meerkat.rest.ProtobufMessageBodyWriter;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.servlet.ServletContainer;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by Laura on 3/20/2017.
 */
public class ReceiverScanHandler implements PollingStationScanner.Consumer{

    public final static String CALLBACKS_ATTRIBUTE_NAME = "callbacks";

    private final Server server;
    private final List<FutureCallback<PollingStation.ScannedData>> callbacks;

    public ReceiverScanHandler(int port, String subAddress) {

        callbacks = new LinkedList<>();

        server = new Server(port);

        ServletContextHandler servletContextHandler = new ServletContextHandler(server, subAddress);
        servletContextHandler.setAttribute(CALLBACKS_ATTRIBUTE_NAME, (Iterable<FutureCallback<PollingStation.ScannedData>>) callbacks);

        ResourceConfig resourceConfig = new ResourceConfig(ReceiverWebAPI.class);
        resourceConfig.register(ProtobufMessageBodyReader.class);
        resourceConfig.register(ProtobufMessageBodyWriter.class);

        ServletHolder servletHolder = new ServletHolder(new ServletContainer(resourceConfig));

        servletContextHandler.addServlet(servletHolder, "/*");
    }

    @Override
    public void start() throws Exception {
        server.start();
    }

    @Override
    public void stop() throws Exception {
        server.stop();
    }

    @Override
    public void subscribe(FutureCallback<PollingStation.ScannedData> scanCallback) {
        callbacks.add(scanCallback);
    }

}